from django.contrib import messages
from django.shortcuts import redirect
from django.utils.translation import gettext_lazy as _
from django.views.generic import FormView

from backoffice.forms import ProfileForm
from backoffice.views.mixins import ConferenceLoginRequiredMixin


class ProfileView(ConferenceLoginRequiredMixin, FormView):
    form_class = ProfileForm
    template_name = 'backoffice/profile.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.update({'instance': self.request.user})
        return kwargs

    def form_valid(self, form):
        form.save()
        messages.success(self.request, _('userprofile_updated'))
        return redirect('backoffice:profile')

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['active_page'] = 'profile'
        return ctx
