__all__ = ('ReportContentView',)

from django_ratelimit.decorators import ratelimit

from django.contrib import messages
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.http import url_has_allowed_host_and_scheme
from django.utils.translation import gettext
from django.views.generic.edit import FormView

from plainui.forms import (
    ReportForm,
)
from plainui.views.utils import (
    ConferenceRequiredMixin,
)


@method_decorator(ratelimit(key='user', rate='1/m', method=ratelimit.UNSAFE), name='dispatch')
@method_decorator(ratelimit(key='ip', rate='5/m', method=ratelimit.UNSAFE), name='dispatch')
class ReportContentView(ConferenceRequiredMixin, FormView):
    require_user = True
    only_published = False
    template_name = 'plainui/report_content.html.j2'
    form_class = ReportForm

    def get_context_data(self, **kwargs):
        return super().get_context_data(conf=self.conf, disable_share=True, **kwargs)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['conf'] = self.conf
        kwargs['request'] = self.request
        return kwargs

    def get_initial(self):
        initial = super().get_initial()
        initial['kind'] = self.kwargs['kind'] if 'kind' in self.kwargs else self.request.GET.get('kind', '')
        initial['kind_data'] = self.kwargs['kind_data'] if 'kind_data' in self.kwargs else self.request.GET.get('kind_data', '')
        initial['lookup_key'] = self.kwargs['lookup_key'] if 'lookup_key' in self.kwargs else self.request.GET.get('lookup_key', '')
        initial['next'] = self.request.GET.get('next', '')
        return initial

    def form_valid(self, form):
        form.send_report_mail(self.request)
        messages.success(
            self.request,
            gettext(
                'Thank you for your help to make this plattform safer and better! '
                'Please give us some time to find a solution and keep an eye on your Messages, we may contact you.'
            ),
        )

        redirect_to = form.cleaned_data['next']
        url_is_safe = url_has_allowed_host_and_scheme(
            url=redirect_to,
            allowed_hosts=[self.request.get_host()],
            require_https=self.request.is_secure(),
        )
        if not url_is_safe:
            redirect_to = reverse('plainui:index')
        return redirect(redirect_to)
